"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeForExport = void 0;
const utils_1 = require("@medusajs/framework/utils");
// We want to have one row per variant, so we need to normalize the data
const normalizeForExport = (product, { regions }) => {
    // Currently region names are treated as case-insensitive.
    const regionsMap = new Map(regions.map((r) => [r.id, r]));
    const res = product.reduce((acc, product) => {
        const variants = product.variants ?? [];
        if (!variants.length) {
            acc.push(normalizeProductForExport(product));
            return acc;
        }
        variants.forEach((v) => {
            const toPush = {
                ...normalizeProductForExport(product),
                ...normalizeVariantForExport(v, regionsMap, product),
            };
            delete toPush["Product Variants"];
            acc.push(toPush);
        });
        return acc;
    }, []);
    return res;
};
exports.normalizeForExport = normalizeForExport;
const normalizeProductForExport = (product) => {
    const flattenedImages = product.images?.reduce((acc, image, idx) => {
        acc[beautifyKey(`product_image_${idx + 1}`)] = image.url;
        return acc;
    }, {});
    const flattenedTags = product.tags?.reduce((acc, tag, idx) => {
        acc[beautifyKey(`product_tag_${idx + 1}`)] = tag.value;
        return acc;
    }, {});
    const flattenedSalesChannels = product.sales_channels?.reduce((acc, salesChannel, idx) => {
        acc[beautifyKey(`product_sales_channel_${idx + 1}`)] = salesChannel.id;
        return acc;
    }, {});
    const flattenedCategories = product.categories?.reduce((acc, category, idx) => {
        acc[beautifyKey(`product_category_${idx + 1}`)] = category.id;
        return acc;
    }, {});
    const res = {
        ...prefixFields(product, "product"),
        ...flattenedImages,
        ...flattenedTags,
        ...flattenedSalesChannels,
        ...flattenedCategories,
    };
    delete res["Product Images"];
    delete res["Product Tags"];
    delete res["Product Sales Channels"];
    delete res["Product Categories"];
    // We can decide if we want the metadata in the export and how that would look like
    delete res["Product Metadata"];
    // We only want the IDs for the type and collection
    delete res["Product Type"];
    delete res["Product Collection"];
    // We just rely on the variant options to reconstruct the product options, so we want to
    // omit the product options to keep the file simpler
    delete res["Product Options"];
    return res;
};
const normalizeVariantForExport = (variant, regionsMap, product) => {
    const flattenedPrices = variant.price_set?.prices
        ?.sort((a, b) => b.currency_code.localeCompare(a.currency_code))
        .reduce((acc, price) => {
        const regionRule = price.price_rules?.find((r) => r.attribute === "region_id");
        if (regionRule) {
            const region = regionsMap.get(regionRule?.value);
            if (!region) {
                throw new utils_1.MedusaError(utils_1.MedusaError.Types.NOT_FOUND, `Region with id ${regionRule?.value} not found`);
            }
            const regionKey = `variant_price_${region.name
                .toLowerCase()
                .split(" ")
                .join("_")}_[${region.currency_code.toUpperCase()}]`;
            acc[beautifyKey(regionKey)] = price.amount;
        }
        else if (!price.price_rules?.length) {
            acc[beautifyKey(`variant_price_${price.currency_code.toUpperCase()}`)] = price.amount;
        }
        return acc;
    }, {});
    const options = product.options ?? [];
    const flattenedOptions = variant.options?.reduce((acc, option, idx) => {
        const prodOptions = options.find((prodOption) => prodOption.id === option.option_id);
        acc[beautifyKey(`variant_option_${idx + 1}_name`)] = prodOptions?.title;
        acc[beautifyKey(`variant_option_${idx + 1}_value`)] = option.value;
        return acc;
    }, {});
    const res = {
        ...prefixFields(variant, "variant"),
        ...flattenedPrices,
        ...flattenedOptions,
    };
    delete res["Variant Price Set"];
    delete res["Variant Options"];
    return res;
};
const prefixFields = (obj, prefix) => {
    const res = {};
    Object.keys(obj).forEach((key) => {
        res[beautifyKey(`${prefix}_${key}`)] = obj[key];
    });
    return res;
};
const beautifyKey = (key) => {
    return key.split("_").map(utils_1.upperCaseFirst).join(" ");
};
//# sourceMappingURL=normalize-for-export.js.map