import { AdditionalData, CreateProductWorkflowInputDTO, ProductTypes } from "@medusajs/framework/types";
import { WorkflowData } from "@medusajs/framework/workflows-sdk";
/**
 * The product's data to validate.
 */
export interface ValidateProductInputStepInput {
    /**
     * The products to validate.
     */
    products: Omit<CreateProductWorkflowInputDTO, "sales_channels">[];
}
/**
 * This step validates that all provided products have options.
 * If a product is missing options or a shipping profile, an error is thrown.
 *
 * @example
 * const data = validateProductInputStep({
 *   products: [
 *     {
 *       title: "Shirt",
 *       options: [
 *         {
 *           title: "Size",
 *           values: ["S", "M", "L"]
 *         }
 *       ],
 *       variants: [
 *         {
 *           title: "Small Shirt",
 *           sku: "SMALLSHIRT",
 *           options: {
 *             Size: "S"
 *           },
 *           prices: [
 *             {
 *               amount: 10,
 *               currency_code: "usd"
 *             }
 *           ],
 *           manage_inventory: true,
 *         },
 *       ]
 *     }
 *   ]
 * })
 */
export declare const validateProductInputStep: import("@medusajs/framework/workflows-sdk").StepFunction<ValidateProductInputStepInput, unknown>;
/**
 * The data to create one or more products, along with custom data that's passed to the workflow's hooks.
 */
export type CreateProductsWorkflowInput = {
    /**
     * The products to create.
     */
    products: CreateProductWorkflowInputDTO[];
} & AdditionalData;
export declare const createProductsWorkflowId = "create-products";
/**
 * This workflow creates one or more products. It's used by the [Create Product Admin API Route](https://docs.medusajs.com/api/admin#products_postproducts).
 * It can also be useful to you when creating [seed scripts](https://docs.medusajs.com/learn/fundamentals/custom-cli-scripts/seed-data), for example.
 *
 * This workflow has a hook that allows you to perform custom actions on the created products. You can see an example in [this guide](https://docs.medusajs.com/resources/commerce-modules/product/extend).
 *
 * You can also use this workflow within your customizations or your own custom workflows, allowing you to wrap custom logic around product creation.
 *
 * :::note
 *
 * Learn more about adding rules to the product variant's prices in the Pricing Module's
 * [Price Rules](https://docs.medusajs.com/resources/commerce-modules/pricing/price-rules) documentation.
 *
 * :::
 *
 * @example
 * const { result } = await createProductsWorkflow(container)
 * .run({
 *   input: {
 *     products: [
 *       {
 *         title: "Shirt",
 *         options: [
 *           {
 *             title: "Size",
 *             values: ["S", "M", "L"]
 *           }
 *         ],
 *         variants: [
 *           {
 *             title: "Small Shirt",
 *             sku: "SMALLSHIRT",
 *             options: {
 *               Size: "S"
 *             },
 *             prices: [
 *               {
 *                 amount: 10,
 *                 currency_code: "usd"
 *               }
 *             ],
 *             manage_inventory: true,
 *           },
 *         ]
 *       }
 *     ],
 *     shipping_profile_id: "sp_123",
 *     additional_data: {
 *       erp_id: "123"
 *     }
 *   }
 * })
 *
 * @summary
 *
 * Create one or more products with options and variants.
 *
 * @property hooks.productsCreated - This hook is executed after the products are created. You can consume this hook to perform custom actions on the created products.
 */
export declare const createProductsWorkflow: import("@medusajs/framework/workflows-sdk").ReturnWorkflow<CreateProductsWorkflowInput, ProductTypes.ProductDTO[], [import("@medusajs/framework/workflows-sdk").Hook<"productsCreated", {
    products: WorkflowData<ProductTypes.ProductDTO[]>;
    additional_data: ((Record<string, unknown> | WorkflowData<Record<string, unknown> | undefined>) & Record<string, unknown>) | undefined;
}, unknown>]>;
//# sourceMappingURL=create-products.d.ts.map