import { AdditionalData, ProductTypes, UpdateProductVariantWorkflowInputDTO } from "@medusajs/framework/types";
import { WorkflowData } from "@medusajs/framework/workflows-sdk";
/**
 * Update products that match a specified selector, along with custom data that's passed to the workflow's hooks.
 */
export type UpdateProductsWorkflowInputSelector = {
    /**
     * The filters to find products to update.
     */
    selector: ProductTypes.FilterableProductProps;
    /**
     * The data to update the products with.
     */
    update: Omit<ProductTypes.UpdateProductDTO, "variants"> & {
        /**
         * The sales channels that the products are available in.
         */
        sales_channels?: {
            id: string;
        }[];
        /**
         * The variants to update.
         */
        variants?: UpdateProductVariantWorkflowInputDTO[];
        /**
         * The shipping profile to set.
         */
        shipping_profile_id?: string | null;
    };
} & AdditionalData;
/**
 * Update one or more products, along with custom data that's passed to the workflow's hooks.
 */
export type UpdateProductsWorkflowInputProducts = {
    /**
     * The products to update.
     */
    products: (Omit<ProductTypes.UpsertProductDTO, "variants"> & {
        /**
         * The sales channels that the products are available in.
         */
        sales_channels?: {
            id: string;
        }[];
        /**
         * The variants to update.
         */
        variants?: UpdateProductVariantWorkflowInputDTO[];
        /**
         * The shipping profile to set.
         */
        shipping_profile_id?: string | null;
    })[];
} & AdditionalData;
/**
 * The data to update one or more products, along with custom data that's passed to the workflow's hooks.
 */
export type UpdateProductWorkflowInput = UpdateProductsWorkflowInputSelector | UpdateProductsWorkflowInputProducts;
export declare const updateProductsWorkflowId = "update-products";
/**
 * This workflow updates one or more products. It's used by the [Update Product Admin API Route](https://docs.medusajs.com/api/admin#products_postproductsid).
 *
 * This workflow has a hook that allows you to perform custom actions on the updated products. For example, you can pass under `additional_data` custom data that
 * allows you to update custom data models linked to the products.
 *
 * You can also use this workflow within your customizations or your own custom workflows, allowing you to wrap custom logic around product update.
 *
 * :::note
 *
 * Learn more about adding rules to the product variant's prices in the Pricing Module's
 * [Price Rules](https://docs.medusajs.com/resources/commerce-modules/pricing/price-rules) documentation.
 *
 * :::
 *
 * @example
 * To update products by their IDs:
 *
 * ```ts
 * const { result } = await updateProductsWorkflow(container)
 * .run({
 *   input: {
 *     products: [
 *       {
 *         id: "prod_123",
 *         title: "Shirts"
 *       },
 *       {
 *         id: "prod_321",
 *         variants: [
 *           {
 *             id: "variant_123",
 *             options: {
 *               Size: "S"
 *             }
 *           }
 *         ]
 *       }
 *     ],
 *     additional_data: {
 *       erp_id: "erp_123"
 *     }
 *   }
 * })
 * ```
 *
 * You can also update products by a selector:
 *
 * ```ts
 * const { result } = await updateProductsWorkflow(container)
 * .run({
 *   input: {
 *     selector: {
 *       type_id: ["ptyp_123"]
 *     },
 *     update: {
 *       description: "This is a shirt product"
 *     },
 *     additional_data: {
 *       erp_id: "erp_123"
 *     }
 *   }
 * })
 * ```
 *
 * @summary
 *
 * Update one or more products with options and variants.
 *
 * @property hooks.productsUpdated - This hook is executed after the products are updated. You can consume this hook to perform custom actions on the updated products.
 */
export declare const updateProductsWorkflow: import("@medusajs/framework/workflows-sdk").ReturnWorkflow<UpdateProductWorkflowInput, ProductTypes.ProductDTO[], [import("@medusajs/framework/workflows-sdk").Hook<"productsUpdated", {
    products: (ProductTypes.ProductDTO | WorkflowData<ProductTypes.ProductDTO>)[] & ProductTypes.ProductDTO[] & import("@medusajs/framework/workflows-sdk").WorkflowDataProperties<ProductTypes.ProductDTO[]> & {
        config(config: {
            name?: string;
        } & Omit<import("@medusajs/orchestration").TransactionStepsDefinition, "next" | "uuid" | "action">): WorkflowData<ProductTypes.ProductDTO[]>;
    } & import("@medusajs/framework/workflows-sdk").StepFunctionReturnConfig<ProductTypes.ProductDTO[]>;
    additional_data: ((Record<string, unknown> | WorkflowData<Record<string, unknown> | undefined>) & Record<string, unknown>) | undefined;
}, unknown>]>;
//# sourceMappingURL=update-products.d.ts.map